from microservice_template_core.settings import FlaskConfig, ServiceConfig, DbConfig
from flask import Flask, Blueprint
from flask_cors import CORS
from prometheus_client import make_wsgi_app
from werkzeug.middleware.dispatcher import DispatcherMiddleware
from prometheus_flask_exporter import PrometheusMetrics


class Core:

    def __init__(self):
        self.config = ServiceConfig.configuration
        self.namespaces = self.config.get('namespaces', [])
        self.blueprint = None
        self.register_namespaces()
        self.app = self.create_flask_app()
        self.ma = None
        if DbConfig.USE_DB:
            self.connect_db()
        if FlaskConfig.FLASK_JWT:
            pass

    def run(self):
        self.app.run(debug=FlaskConfig.FLASK_DEBUG,
                     port=ServiceConfig.SERVICE_PORT,
                     host=ServiceConfig.SERVER_NAME,
                     threaded=FlaskConfig.FLASK_THREADED
                     )

    def register_namespaces(self):
        from microservice_template_core.tools.flask_restplus import api
        self.blueprint = Blueprint('api', __name__,
                                   url_prefix=ServiceConfig.URL_PREFIX)
        api.init_app(self.blueprint)
        from microservice_template_core.api.namespaces.core_endpoints import ns as core_endpoints
        default_namespaces = [core_endpoints]

        for namespace in default_namespaces + self.namespaces:
            api.add_namespace(namespace)

    def connect_db(self):
        from microservice_template_core.tools.db import db
        db.init_app(self.app)
        db.app = self.app
        db.create_all()

    def add_authorization(self):
        from flask_jwt_extended import JWTManager
        if FlaskConfig.FLASK_JWT_SECRET_KEY:
            self.app.config['JWT_SECRET_KEY'] = FlaskConfig.FLASK_JWT_SECRET_KEY
            JWTManager(self.app)

    def create_flask_app(self):
        app = Flask(__name__)
        metrics = PrometheusMetrics(app)
        CORS(app)
        app.register_blueprint(self.blueprint)
        app.wsgi_app = DispatcherMiddleware(app.wsgi_app, {'/metrics': make_wsgi_app()})
        app.config['SQLALCHEMY_DATABASE_URI'] = DbConfig.SQLALCHEMY_DATABASE_URI
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = DbConfig.SQLALCHEMY_TRACK_MODIFICATIONS
        app.config['SQLALCHEMY_POOL_RECYCLE'] = DbConfig.SQLALCHEMY_POOL_RECYCLE
        return app

    def configure_opentelemetry_client(self):
        pass

    def configure_prometheus_metrics(self):
        pass

    def configure_db(self):
        pass


def main():
    microservice_template_core = Core()
    microservice_template_core.run()


if __name__ == '__main__':
    main()



from opentelemetry import trace
from opentelemetry.exporter import jaeger
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchExportSpanProcessor
from microservice_template_core.settings import TracerConfig, ServiceConfig
tracer = None


def get_tracer():
    global tracer
    if not tracer:
        trace.set_tracer_provider(TracerProvider())
        jaeger_exporter = jaeger.JaegerSpanExporter(
            service_name=ServiceConfig.SERVICE_NAME,
            agent_host_name=TracerConfig.JAEGER_SERVER,
            agent_port=TracerConfig.JAEGER_PORT,
        )

        trace.get_tracer_provider().add_span_processor(
            BatchExportSpanProcessor(jaeger_exporter)
        )

        tracer = trace.get_tracer(__name__)
    return tracer



import logging
import logging_loki
import datetime
from microservice_template_core.settings import LoggerConfig, ServiceConfig
from multiprocessing import Queue


logger = None


class VerboseFilter(logging.Filter):

    def filter(self, record: logging.LogRecord) -> bool:
        if not hasattr(record, 'traceback'):
            record.traceback = ""
        return True


class VerboseFormatter(logging.Formatter):

    def formatMessage(self, record: logging.LogRecord) -> str:
        base_message = f'{datetime.datetime.now()} - {record.name} - {record.levelname} - {record.message}'
        if hasattr(record, 'exception'):
            base_message = base_message + f'\nException: {getattr(record, "exception")}'
        if hasattr(record, 'traceback'):
            base_message = base_message + f'\nTraceback: {getattr(record, "traceback")}'

        return base_message


def get_logger():
    global logger
    if not logger:
        logger = logging.getLogger(ServiceConfig.SERVICE_NAME)
        logger.setLevel(LoggerConfig.LOG_LEVEL)
        # loki_handler = logging_loki.LokiQueueHandler(
        #     Queue(-1),
        #     url=f"http://{LoggerConfig.LOKI_SERVER}:{LoggerConfig.LOKI_PORT}/loki/api/v1/push",
        #     tags={"service": ServiceConfig.SERVICE_NAME},
        #     version="1",
        # )
        console_handler = logging.StreamHandler()
        console_format = logging.Formatter('[%(asctime)s] {%(module)s} [%(levelname)s] - %(message)s')
        console_handler.setFormatter(console_format)

        if LoggerConfig.LOGGING_VERBOSE:
            print("Verbose logger initialization")
            console_handler = logging.StreamHandler()
            console_handler.setLevel("DEBUG")
            console_handler.setFormatter(VerboseFormatter())
            logger.addHandler(console_handler)
            logger.addFilter(VerboseFilter())
        # logger.addHandler(loki_handler)
        logger.addHandler(console_handler)
        logger.info(
            "Service logger initialization",
            extra={"tags": {"tests": "logger-test"}},
        )
    return logger


if __name__ == '__main__':
    LoggerConfig.LOGGING_VERBOSE = True
    test_logger = get_logger()
    test_logger.critical(
        msg="It's test message to check if logging works",
        extra={
            "tags": {
                "tests": "logger-test",
                "traceback": "Traceback here",
                "exception": "It's test exception"
            }
        }
    )



from .global_settings import FlaskConfig
from .global_settings import ServiceConfig
from .global_settings import LoggerConfig
from .global_settings import TracerConfig
from .global_settings import DbConfig



import os


class ServiceConfig:
    SERVICE_NAME = os.getenv('SERVICE_NAME', 'microservice_template_core')
    SERVICE_PORT = os.getenv('SERVICE_PORT', 8081)
    SERVER_NAME = os.getenv('SERVER_NAME', "0.0.0.0")
    URL_PREFIX = os.getenv('URL_PREFIX', '/api/v1')
    configuration = {}


class LoggerConfig:
    LOG_LEVEL = os.getenv('LOG_LEVEL', 'DEBUG')
    LOGGING_VERBOSE = os.getenv('LOGGING_VERBOSE', False)
    LOKI_SERVER = os.getenv('LOKI_SERVER', '127.0.0.1')
    LOKI_PORT = os.getenv('LOKI_PORT', 3100)


class TracerConfig:
    JAEGER_SERVER = os.getenv('JAEGER_SERVER', '127.0.0.1')
    JAEGER_PORT = int(os.getenv('JAEGER_PORT', 6831))


class FlaskConfig:
    FLASK_DEBUG = os.getenv('FLASK_DEBUG', False)
    FLASK_THREADED = os.getenv('FLASK_THREADED', False)
    FLASK_JWT = os.getenv('FLASK_JWT', False)
    FLASK_JWT_SECRET_KEY = os.getenv('FLASK_JWT_SECRET_KEY', None)


class DbConfig:
    USE_DB = os.getenv('USE_DB', False)
    DATABASE_SERVER = os.getenv('DATABASE_SERVER', '127.0.0.1')
    DATABASE_PORT = os.getenv('DATABASE_PORT', 3306)
    DATABASE_USER = os.getenv('DATABASE_USER', 'microservice_template_core')
    DATABASE_PSWD = os.getenv('DATABASE_PSWD', 'microservice_template_core')
    DATABASE_SCHEMA = os.getenv('DATABASE_SCHEMA', 'harp')
    SQLALCHEMY_DATABASE_URI = f'mysql+pymysql://{DATABASE_USER}:{DATABASE_PSWD}@' \
                              f'{DATABASE_SERVER}:{DATABASE_PORT}/{DATABASE_SCHEMA}'
    SQLALCHEMY_TRACK_MODIFICATIONS = os.getenv('SQLALCHEMY_TRACK_MODIFICATIONS', False)
    SQLALCHEMY_POOL_RECYCLE = os.getenv('', 300)

